import { Behavior, FASTElement } from "@microsoft/fast-element";
export interface CSSCustomPropertyDefinition {
    /**
     * The custom property name
     */
    name: string;
    /**
     * The value of the custom property or a function that resolves the value
     */
    value: string | ((...args: any[]) => string);
}
export interface CSSCustomPropertyTarget {
    registerCSSCustomProperty(behavior: CSSCustomPropertyDefinition): void;
    unregisterCSSCustomProperty(behavior: CSSCustomPropertyDefinition): void;
    disconnectedCSSCustomPropertyRegistry: CSSCustomPropertyDefinition[] | void;
}
/**
 * A Behavior that, when bound, will register itself with the nearest
 * CSSCustomPropertyTarget
 */
export declare class CSSCustomPropertyBehavior implements Behavior, CSSCustomPropertyDefinition {
    /**
     * The name of the CSS Custom Property behavior.
     */
    readonly name: CSSCustomPropertyDefinition["name"];
    /**
     * The value or function that will resolve the value of
     * the CSS Custom Property.
     */
    readonly value: CSSCustomPropertyDefinition["value"];
    /**
     * The name of the CSSCustomPropertyBehavior formatted
     * as a CSS custom property.
     */
    readonly propertyName: string;
    /**
     * Return the CSS Custom Property formatted
     * as a CSS variable.
     */
    readonly var: string;
    constructor(
    /**
     * The name of the custom property.
     * @remarks
     * When written as a custom property, the "--" required
     * by CSS custom properties will be prepended to this value.
     */
    name: string, 
    /**
     * The value of the custom property or a function that
     * resolves the value.
     * @remarks
     * When a resolver is provided, the resolver will be invoked
     * by the element returned from the host argument.
     */
    value: CSSCustomPropertyDefinition["value"], 
    /**
     * The element that registers the behavior.
     * @remarks
     * This element should also be responsible for resolving
     * and function value.
     */
    host: (source: HTMLElement) => Partial<CSSCustomPropertyTarget> | null);
    private host;
    /**
     * Binds the behavior to a source element
     * @param source The source element being bound
     * @internal
     */
    bind(source: HTMLElement): void;
    /**
     * Unbinds the behavior from the source element.
     * @param source The source element being unbound
     * @internal
     */
    unbind(source: HTMLElement): void;
}
/**
 * Create a CSS Custom Property behavior.
 * @param name The name of the CSS custom property
 * @param value The value or value resolver of the custom property
 * @param host A function to resolve the element to host the CSS custom property
 */
export declare function cssCustomPropertyBehaviorFactory(name: string, value: string | ((...arg: any[]) => string), host: (source: typeof FASTElement & HTMLElement) => Partial<CSSCustomPropertyTarget> | null): CSSCustomPropertyBehavior;
