import { Behavior, FASTElement } from "@microsoft/fast-element";
import { PartialFASTElementDefinition } from "@microsoft/fast-element";
import { CSSCustomPropertyDefinition, CSSCustomPropertyTarget } from "../custom-properties/index";
import { DecoratorDesignSystemPropertyConfiguration } from "./design-system-property";
export interface DesignSystemConsumer {
    provider: DesignSystemProvider | null;
}
/**
 * Determines if the element has a design-system-provider context
 * @param element
 */
export declare function isDesignSystemConsumer(element: HTMLElement | DesignSystemConsumer): element is DesignSystemConsumer;
/**
 * Behavior to connect an element to the nearest design-system provider
 */
export declare const designSystemConsumerBehavior: Behavior;
export declare class DesignSystemProvider extends FASTElement implements CSSCustomPropertyTarget, DesignSystemConsumer {
    /**
     * Stores a list of all element tag-names that associated
     * to design-system-providers
     */
    private static _tagNames;
    /**
     * Read all tag-names that are associated to
     * design-system-providers
     */
    static get tagNames(): string[];
    /**
     * Determines if an element is a DesignSystemProvider
     * @param el The element to test
     */
    static isDesignSystemProvider(el: HTMLElement | DesignSystemProvider): el is DesignSystemProvider;
    /**
     * Finds the closest design-system-provider
     * to an element.
     */
    static findProvider(el: HTMLElement & Partial<DesignSystemConsumer>): DesignSystemProvider | null;
    /**
     * Registers a tag-name to be associated with
     * the design-system-provider class
     */
    static registerTagName(tagName: string): void;
    /**
     * Allows other components to identify this as a provider.
     * Using instanceof DesignSystemProvider did not seem to work.
     */
    readonly isDesignSystemProvider = true;
    /**
     * The design-system object.
     * This is "observable" but will notify on object mutation
     * instead of object assignment
     */
    designSystem: {};
    /**
     * Applies the default design-system values to the instance where properties
     * are not explicitly assigned. This is generally used to set the root design
     * system context.
     */
    useDefaults: boolean;
    private useDefaultsChanged;
    provider: DesignSystemProvider | null;
    private providerChanged;
    /**
     * Stores all CSSCustomPropertyDefinitions registered with the provider.
     */
    private cssCustomPropertyDefinitions;
    /**
     * Track all design system property names so we can react to changes
     * in those properties. Do not initialize or it will clobber value stored
     * by the decorator.
     */
    designSystemProperties: {
        [propertyName: string]: Required<Pick<DecoratorDesignSystemPropertyConfiguration, "cssCustomProperty" | "default">>;
    };
    /**
     * Allows CSSCustomPropertyDefinitions to register on this element *before* the constructor
     * has run and the registration APIs exist. This can manifest when the DOM
     * is parsed (and custom element tags exist in the DOM) before the script defining the custom elements
     * and elements is parsed, and the elements using the CSSCustomPropertyBehaviors
     * are defined before this DesignSystemProvider.
     */
    disconnectedCSSCustomPropertyRegistry: CSSCustomPropertyDefinition[];
    /**
     * The target of CSSCustomPropertyDefinitions registered
     * with the provider. This will be #1 when adoptedStyleSheets are supported
     * and #2 when they are not.
     *
     * 1. The `style` property of a CSSStyleRule on an adoptedStyleSheet
     * 2. The `style` property of the element, resulting in inline styles
     */
    private customPropertyTarget;
    /**
     * Handle changes to design-system-provider IDL and content attributes
     * that reflect to design-system properties.
     */
    private attributeChangeHandler;
    /**
     * Handle changes to the local design-system property.
     */
    private localDesignSystemChangeHandler;
    /**
     * Handle changes to the upstream design-system provider
     */
    private providerDesignSystemChangeHandler;
    constructor();
    connectedCallback(): void;
    /**
     * Register a CSSCustomPropertyDefinition with the design system provider.
     * Registering a CSSCustomPropertyDefinition will create the CSS custom property.
     */
    registerCSSCustomProperty(behavior: CSSCustomPropertyDefinition): void;
    /**
     * Unregister a CSSCustomPropertyDefinition. If all registrations of the definition
     * are unregistered, the CSS custom property will be removed.
     */
    unregisterCSSCustomProperty(behavior: CSSCustomPropertyDefinition): void;
    /**
     * Writes all CSS custom property definitions to the design system provider.
     */
    private writeCustomProperties;
    /**
     * Writes a CSS custom property to the design system provider,
     * evaluating any function values with the design system.
     */
    private setCustomProperty;
    /**
     * Removes a CSS custom property from the provider.
     */
    private deleteCustomProperty;
    /**
     * Evaluates a CSSCustomPropertyDefinition with the current design system.
     */
    evaluate(definition: CSSCustomPropertyDefinition): string;
    /**
     * Synchronize the provider's design system with the local
     * overrides. Any value defined on the instance will take priority
     * over the value defined by the provider
     */
    private syncDesignSystemWithProvider;
    private isValidDesignSystemValue;
}
/**
 * Defines a design-system-provider custom element
 */
export declare function designSystemProvider(nameOrDef: string | PartialFASTElementDefinition): <T extends typeof DesignSystemProvider>(providerCtor: T) => void;
