import { FASTElement } from "@microsoft/fast-element";
/**
 * This file enables typing support for ElementInternals APIs.
 * It is largely taken from https://github.com/microsoft/TSJS-lib-generator/pull/818/files.
 *
 * When TypeScript adds support for these APIs we can delete this file.
 */
interface ValidityStateFlags {
    badInput?: boolean;
    customError?: boolean;
    patternMismatch?: boolean;
    rangeOverflow?: boolean;
    rangeUnderflow?: boolean;
    stepMismatch?: boolean;
    tooLong?: boolean;
    tooShort?: boolean;
    typeMismatch?: boolean;
    valueMissing?: boolean;
}
/**
 * Source:
 * https://html.spec.whatwg.org/multipage/custom-elements.html#elementinternals
 */
interface ElementInternals {
    /**
     * Returns the form owner of internals target element.
     */
    readonly form: HTMLFormElement | null;
    /**
     * Returns a NodeList of all the label elements that internals target element is associated with.
     */
    readonly labels: NodeList;
    /**
     * Returns the error message that would be shown to the user if internals target element was to be checked for validity.
     */
    readonly validationMessage: string;
    /**
     * Returns the ValidityState object for internals target element.
     */
    readonly validity: ValidityState;
    /**
     * Returns true if internals target element will be validated when the form is submitted; false otherwise.
     */
    readonly willValidate: boolean;
    /**
     * Returns true if internals target element has no validity problems; false otherwise. Fires an invalid event at the element in the latter case.
     */
    checkValidity(): boolean;
    /**
     * Returns true if internals target element has no validity problems; otherwise,
     * returns false, fires an invalid event at the element, and (if the event isn't canceled) reports the problem to the user.
     */
    reportValidity(): boolean;
    /**
     * Sets both the state and submission value of internals target element to value.
     *
     * While "null" isn't enumerated as a argument type (here)[https://html.spec.whatwg.org/multipage/custom-elements.html#the-elementinternals-interface],
     * In practice it appears to remove the value from the form data on submission. Adding it as a valid type here
     * becuase that capability is required for checkbox and radio types
     */
    setFormValue(value: File | string | FormData | null, state?: File | string | FormData | null): void;
    /**
     * Marks internals target element as suffering from the constraints indicated by the flags argument,
     * and sets the element's validation message to message.
     * If anchor is specified, the user agent might use
     * it to indicate problems with the constraints of internals target
     * element when the form owner is validated interactively or reportValidity() is called.
     */
    setValidity(flags: ValidityStateFlags, message?: string, anchor?: HTMLElement): void;
}
declare let ElementInternals: {
    prototype: ElementInternals;
    new (): ElementInternals;
};
interface HTMLElement {
    attachInternals?(): ElementInternals;
}
export declare const supportsElementInternals: boolean;
/**
 * Disable member ordering to keep property callbacks
 * grouped with property declaration
 */
export declare abstract class FormAssociated<T extends HTMLInputElement | HTMLTextAreaElement | HTMLSelectElement> extends FASTElement {
    /**
     * Must evaluate to true to enable elementInternals.
     * Feature detects API support and resolve respectively
     */
    static get formAssociated(): boolean;
    /**
     * Returns the validty state of the element
     */
    get validity(): ValidityState;
    /**
     * Retrieve a reference to the associated form.
     * Returns null if not associated to any form.
     */
    get form(): HTMLFormElement | null;
    /**
     * Retrieve the localized validation message,
     * or custom validation message if set.
     */
    get validationMessage(): string;
    /**
     * Whether the element will be validated when the
     * form is submitted
     */
    get willValidate(): boolean;
    /**
     * A reference to all associated label elements
     */
    get labels(): ReadonlyArray<Node>;
    /**
     * The value of the element to be associated with the form
     */
    value: string;
    disabled: boolean;
    protected disabledChanged(): void;
    name: string;
    protected nameChanged(): void;
    /**
     * Require the field prior to form submission
     */
    required: boolean;
    protected requiredChanged(): void;
    /**
     * The proxy element provided by
     */
    protected abstract proxy: T;
    /**
     * The element internals object. Will only exist
     * in browsers supporting the attachInternals API
     */
    protected elementInternals: ElementInternals;
    /**
     * These are events that are still fired by the proxy
     * element based on user / programmatic interaction.
     *
     * The proxy implementation should be transparent to
     * the app author, so block these events from emitting.
     */
    private proxyEventsToBlock;
    constructor();
    connectedCallback(): void;
    disconnectedCallback(): void;
    /**
     * Return the current validity of the element
     */
    checkValidity(): boolean;
    /**
     * Return the current validity of the element.
     * If false, fires an invalid event at the element.
     */
    reportValidity(): boolean;
    /**
     * Set the validity of the control. In cases when the elementInternals object is not
     * available (and the proxy element is used to report validity), this function will
     * do nothing unless a message is provided, at which point the setCustomValidity method
     * of the proxy element will be invoked with the provided message.
     * @param flags Validity flags
     * @param message Optional message to supply
     * @param anchor Optional element used by UA to display an interactive validation UI
     */
    setValidity(flags: ValidityStateFlags, message?: string, anchor?: HTMLElement): void;
    /**
     * Invoked when a connected component's form or fieldset has it's disabled
     * state changed.
     * @param disabled the disabled value of the form / fieldset
     */
    formDisabledCallback(disabled: boolean): void;
    /**
     *
     * @param value The value to set
     * @param state The state object provided to during session restores and when autofilling.
     */
    protected setFormValue(value: File | string | FormData | null, state?: File | string | FormData | null): void;
    protected keypressHandler(e: KeyboardEvent): void;
    /**
     * Used to stop propagation of proxy element events
     * @param e Event object
     */
    private stopPropagation;
}
export {};
