/**
 * A Behavior that, when bound, will register itself with the nearest
 * CSSCustomPropertyTarget
 */
export class CSSCustomPropertyBehavior {
    constructor(
    /**
     * The name of the custom property.
     * @remarks
     * When written as a custom property, the "--" required
     * by CSS custom properties will be prepended to this value.
     */
    name, 
    /**
     * The value of the custom property or a function that
     * resolves the value.
     * @remarks
     * When a resolver is provided, the resolver will be invoked
     * by the element returned from the host argument.
     */
    value, 
    /**
     * The element that registers the behavior.
     * @remarks
     * This element should also be responsible for resolving
     * and function value.
     */
    host) {
        this.name = name;
        this.value = value;
        this.host = host;
        this.propertyName = `--${name}`;
        this.var = `var(${this.propertyName})`;
    }
    /**
     * Binds the behavior to a source element
     * @param source The source element being bound
     * @internal
     */
    bind(source) {
        const target = this.host(source);
        if (target !== null) {
            if (typeof target.registerCSSCustomProperty === "function") {
                target.registerCSSCustomProperty(this);
            }
            else {
                // There is potential for the custom property host element to not be
                // constructed when this is run. We handle that case by accumulating
                // the behaviors in a normal array. Behaviors associated this way will
                // get registered when the host is connected
                if (!Array.isArray(target.disconnectedCSSCustomPropertyRegistry)) {
                    target.disconnectedCSSCustomPropertyRegistry = [];
                }
                target.disconnectedCSSCustomPropertyRegistry.push(this);
            }
        }
    }
    /**
     * Unbinds the behavior from the source element.
     * @param source The source element being unbound
     * @internal
     */
    unbind(source) {
        const target = this.host(source);
        if (target !== null && typeof target.unregisterCSSCustomProperty === "function") {
            target.unregisterCSSCustomProperty(this);
        }
    }
}
/**
 * Create a CSS Custom Property behavior.
 * @param name The name of the CSS custom property
 * @param value The value or value resolver of the custom property
 * @param host A function to resolve the element to host the CSS custom property
 */
export function cssCustomPropertyBehaviorFactory(name, value, host) {
    return new CSSCustomPropertyBehavior(name, value, host);
}
