import { __decorate } from "tslib";
import { FASTElement, observable } from "@microsoft/fast-element";
import { inRange, invert } from "lodash-es";
import { isHTMLElement, keyCodeArrowDown, keyCodeArrowLeft, keyCodeArrowRight, keyCodeArrowUp, keyCodeEnd, keyCodeHome, } from "@microsoft/fast-web-utilities";
import { MenuItemRole } from "../menu-item/index";
export class Menu extends FASTElement {
    constructor() {
        super(...arguments);
        /**
         * The index of the focusable element in the items array
         * defaults to -1
         */
        this.focusIndex = -1;
        /**
         * if focus is moving out of the menu, reset to a stable initial state
         */
        this.handleFocusOut = (e) => {
            const isNestedEl = this.contains(e.relatedTarget);
            if (!isNestedEl) {
                // find our first focusable element
                const focusIndex = this.menuItems.findIndex(this.isFocusableElement);
                // set the current focus index's tabindex to -1
                this.menuItems[this.focusIndex].setAttribute("tabindex", "");
                // set the first focusable element tabindex to 0
                this.menuItems[focusIndex].setAttribute("tabindex", "0");
                // set the focus index
                this.focusIndex = focusIndex;
            }
        };
        this.setItems = () => {
            const focusIndex = this.menuItems.findIndex(this.isFocusableElement);
            // if our focus index is not -1 we have items
            if (focusIndex !== -1) {
                this.focusIndex = focusIndex;
            }
            for (let item = 0; item < this.menuItems.length; item++) {
                if (item === focusIndex) {
                    this.menuItems[item].setAttribute("tabindex", "0");
                }
                this.menuItems[item].addEventListener("blur", this.handleMenuItemFocus);
            }
        };
        this.resetItems = (oldValue) => {
            for (let item = 0; item < oldValue.length; item++) {
                oldValue[item].removeEventListener("blur", this.handleMenuItemFocus);
            }
        };
        /**
         * check if the item is a menu item
         */
        this.isMenuItemElement = (el) => {
            return (isHTMLElement(el) &&
                Menu.focusableElementRoles.hasOwnProperty(el.getAttribute("role")));
        };
        /**
         * check if the item is disabled
         */
        this.isDisabledElement = (el) => {
            return this.isMenuItemElement(el) && el.getAttribute("aria-disabled") === "true";
        };
        /**
         * check if the item is focusable
         */
        this.isFocusableElement = (el) => {
            return this.isMenuItemElement(el) && !this.isDisabledElement(el);
        };
        this.handleMenuItemFocus = (e) => {
            const target = e.currentTarget;
            const focusIndex = this.menuItems.indexOf(target);
            if (this.isDisabledElement(target)) {
                target.blur();
                return;
            }
            if (focusIndex !== this.focusIndex && focusIndex !== -1) {
                this.setFocus(focusIndex, focusIndex > this.focusIndex ? 1 : -1);
            }
        };
    }
    itemsChanged(oldValue, newValue) {
        if (this.$fastController.isConnected) {
            this.menuItems = this.domChildren();
            this.resetItems(oldValue);
            this.setItems();
        }
    }
    connectedCallback() {
        super.connectedCallback();
        this.menuItems = this.domChildren();
        this.setItems();
    }
    focus() {
        this.setFocus(0, 1);
    }
    handleMenuKeyDown(e) {
        switch (e.keyCode) {
            case keyCodeArrowDown:
            case keyCodeArrowRight:
                // go forward one index
                e.preventDefault();
                this.setFocus(this.focusIndex + 1, 1);
                break;
            case keyCodeArrowUp:
            case keyCodeArrowLeft:
                // go back one index
                e.preventDefault();
                this.setFocus(this.focusIndex - 1, -1);
                break;
            case keyCodeEnd:
                // set focus on last item
                e.preventDefault();
                this.setFocus(this.domChildren().length - 1, -1);
                break;
            case keyCodeHome:
                // set focus on first item
                e.preventDefault();
                this.setFocus(0, 1);
                break;
            default:
                // if we are not handling the event, do not prevent default
                return true;
        }
    }
    /**
     * get an array of valid DOM children
     */
    domChildren() {
        return Array.from(this.children);
    }
    setFocus(focusIndex, adjustment) {
        const children = this.menuItems;
        while (inRange(focusIndex, children.length)) {
            const child = children[focusIndex];
            if (this.isFocusableElement(child)) {
                // update the tabindex of next focusable element
                child.setAttribute("tabindex", "0");
                // focus the element
                child.focus();
                // change the previous index to -1
                children[this.focusIndex].setAttribute("tabindex", "");
                // update the focus index
                this.focusIndex = focusIndex;
                break;
            }
            focusIndex += adjustment;
        }
    }
}
Menu.focusableElementRoles = invert(MenuItemRole);
__decorate([
    observable
], Menu.prototype, "items", void 0);
