import { __decorate } from "tslib";
import { attr, nullableNumberConverter, observable } from "@microsoft/fast-element";
import { Direction, keyCodeArrowDown, keyCodeArrowLeft, keyCodeArrowRight, keyCodeArrowUp, keyCodeEnd, keyCodeHome, keyCodeTab, Orientation, } from "@microsoft/fast-web-utilities";
import { FormAssociated } from "../form-associated/index";
import { convertPixelToPercent } from "./slider-utilities";
export var SliderMode;
(function (SliderMode) {
    SliderMode["singleValue"] = "single-value";
})(SliderMode || (SliderMode = {}));
export class Slider extends FormAssociated {
    constructor() {
        super();
        this.direction = Direction.ltr;
        this.isDragging = false;
        this.trackWidth = 0;
        this.trackMinWidth = 0;
        this.trackHeight = 0;
        this.trackMinHeight = 0;
        /**
         * Min allowed value default is 0
         */
        this.min = 0; // Map to proxy element.
        /**
         * Max allowed value default is 10
         */
        this.max = 10; // Map to proxy element.
        /**
         * Value to increment or decrement via arrow keys, mouse click or drag
         */
        this.step = 1; // Map to proxy element.
        /**
         * Orientation value, horizontal | vertical
         */
        this.orientation = Orientation.horizontal;
        /**
         * mode value, default singleValue
         */
        this.mode = SliderMode.singleValue;
        this.proxy = document.createElement("input");
        this.increment = () => {
            const newVal = this.direction !== Direction.rtl && this.orientation !== Orientation.vertical
                ? Number(this.value) + Number(this.step)
                : Number(this.value) - Number(this.step);
            const incrementedVal = this.convertToConstrainedValue(newVal);
            const incrementedValString = incrementedVal < Number(this.max) ? `${incrementedVal}` : `${this.max}`;
            this.value = incrementedValString;
            this.updateForm();
        };
        this.decrement = () => {
            const newVal = this.direction !== Direction.rtl && this.orientation !== Orientation.vertical
                ? Number(this.value) - Number(this.step)
                : Number(this.value) + Number(this.step);
            const decrementedVal = this.convertToConstrainedValue(newVal);
            const decrementedValString = decrementedVal > Number(this.min) ? `${decrementedVal}` : `${this.min}`;
            this.value = decrementedValString;
            this.updateForm();
        };
        this.keypressHandler = (e) => {
            super.keypressHandler(e);
            if (e.keyCode !== keyCodeTab) {
                e.preventDefault();
            }
            if (e.keyCode === keyCodeHome) {
                this.value = `${this.min}`;
            }
            else if (e.keyCode === keyCodeEnd) {
                this.value = `${this.max}`;
            }
            else if (!e.shiftKey) {
                switch (e.keyCode) {
                    case keyCodeArrowRight:
                    case keyCodeArrowUp:
                        this.increment();
                        break;
                    case keyCodeArrowLeft:
                    case keyCodeArrowDown:
                        this.decrement();
                        break;
                }
            }
        };
        this.setThumbPositionForOrientation = (direction) => {
            const newPct = convertPixelToPercent(Number(this.value), Number(this.min), Number(this.max), direction);
            const percentage = (1 - newPct) * 100;
            if (this.orientation === Orientation.horizontal) {
                this.position = this.isDragging
                    ? `right: ${percentage}%; transition: all 0.1s ease;`
                    : `right: ${percentage}%; transition: all 0.2s ease;`;
            }
            else {
                this.position = this.isDragging
                    ? `bottom: ${percentage}%; transition: all 0.1s ease;`
                    : `bottom: ${percentage}%; transition: all 0.2s ease;`;
            }
        };
        this.getDirection = () => {
            const dirNode = this.parentElement.closest("[dir]");
            if (dirNode && dirNode.dir === "rtl") {
                this.setThumbPositionForOrientation(Direction.rtl);
            }
            return dirNode !== null && dirNode.dir === "rtl" ? Direction.rtl : Direction.ltr;
        };
        this.setupTrackConstraints = () => {
            this.trackWidth = this.track.clientWidth;
            this.trackMinWidth = this.track.clientLeft;
            const clientRect = this.track.getBoundingClientRect();
            this.trackHeight = clientRect.bottom;
            this.trackMinHeight = clientRect.top;
        };
        this.setupListeners = () => {
            this.addEventListener("keydown", this.keypressHandler);
            this.addEventListener("mousedown", this.clickHandler);
            this.thumb.addEventListener("mousedown", this.handleThumbMouseDown);
        };
        this.setupDefaultValue = () => {
            if (this.value === "") {
                this.value = `${this.convertToConstrainedValue((this.max + this.min) / 2)}`;
                this.updateForm();
            }
        };
        this.updateForm = () => {
            this.setFormValue(this.value, this.value);
        };
        /**
         *  Handle mouse moves during a thumb drag operation
         */
        this.handleThumbMouseDown = (event) => {
            if (this.readOnly || this.disabled || event.defaultPrevented) {
                return;
            }
            event.preventDefault();
            event.target.focus();
            window.addEventListener("mouseup", this.handleWindowMouseUp);
            window.addEventListener("mousemove", this.handleMouseMove);
            this.isDragging = true;
        };
        /**
         *  Handle mouse moves during a thumb drag operation
         */
        this.handleMouseMove = (e) => {
            if (this.readOnly || this.disabled || e.defaultPrevented) {
                return;
            }
            // update the value based on current position
            const eventValue = this.orientation === Orientation.horizontal
                ? e.pageX - this.getBoundingClientRect().left
                : e.pageY;
            this.value = `${this.calculateNewValue(eventValue)}`;
            this.updateForm();
        };
        this.calculateNewValue = (rawValue) => {
            // update the value based on current position
            const newPosition = convertPixelToPercent(rawValue, this.orientation === Orientation.horizontal
                ? this.trackMinWidth
                : this.trackMinHeight, this.orientation === Orientation.horizontal
                ? this.trackWidth
                : this.trackHeight, this.direction);
            const newValue = (this.max - this.min) * newPosition + this.min;
            return this.convertToConstrainedValue(newValue);
        };
        /**
         * Handle a window mouse up during a drag operation
         */
        /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
        this.handleWindowMouseUp = (event) => {
            this.stopDragging();
        };
        this.stopDragging = () => {
            this.isDragging = false;
            window.removeEventListener("mouseup", this.handleWindowMouseUp);
            window.removeEventListener("mousemove", this.handleMouseMove);
        };
        this.clickHandler = (e) => {
            e.preventDefault();
            if (!this.disabled && !this.readOnly) {
                this.trackWidth = this.track.clientWidth;
                if (this.trackWidth === 0) {
                    this.trackWidth = 1;
                }
                e.target.focus();
                window.addEventListener("mouseup", this.handleWindowMouseUp);
                window.addEventListener("mousemove", this.handleMouseMove);
                const controlValue = this.orientation === Orientation.horizontal
                    ? e.pageX - this.getBoundingClientRect().left
                    : e.pageY;
                this.value = `${this.calculateNewValue(controlValue)}`;
                this.updateForm();
            }
        };
        this.convertToConstrainedValue = (value) => {
            let constrainedValue = value - this.min;
            const remainderVal = constrainedValue % Number(this.step);
            constrainedValue =
                remainderVal >= Number(this.step) / 2
                    ? constrainedValue - remainderVal + Number(this.step)
                    : constrainedValue - remainderVal;
            return constrainedValue + this.min;
        };
        this.proxy.setAttribute("type", "range");
    }
    readOnlyChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.readOnly = this.readOnly;
        }
    }
    valueChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.updateForm();
        }
        if (this.$fastController.isConnected) {
            this.setThumbPositionForOrientation(this.direction);
        }
        this.$emit("change");
    }
    minChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.min = `${this.min}`;
        }
    }
    maxChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.max = `${this.max}`;
        }
    }
    stepChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.step = `${this.step}`;
        }
    }
    orientationChanged() {
        if (this.$fastController.isConnected) {
            this.setThumbPositionForOrientation(this.direction);
        }
    }
    connectedCallback() {
        super.connectedCallback();
        this.direction = this.getDirection();
        this.updateForm();
        this.setupTrackConstraints();
        this.setupListeners();
        this.setupDefaultValue();
        this.setThumbPositionForOrientation(this.direction);
    }
    disconnectedCallback() {
        this.removeEventListener("keydown", this.keypressHandler);
        this.removeEventListener("mousedown", this.clickHandler);
        this.thumb.removeEventListener("mousedown", this.handleThumbMouseDown);
    }
}
__decorate([
    attr({ attribute: "readonly", mode: "boolean" })
], Slider.prototype, "readOnly", void 0);
__decorate([
    observable
], Slider.prototype, "direction", void 0);
__decorate([
    observable
], Slider.prototype, "isDragging", void 0);
__decorate([
    observable
], Slider.prototype, "position", void 0);
__decorate([
    observable
], Slider.prototype, "trackWidth", void 0);
__decorate([
    observable
], Slider.prototype, "trackMinWidth", void 0);
__decorate([
    observable
], Slider.prototype, "trackHeight", void 0);
__decorate([
    observable
], Slider.prototype, "trackMinHeight", void 0);
__decorate([
    attr({ converter: nullableNumberConverter })
], Slider.prototype, "min", void 0);
__decorate([
    attr({ converter: nullableNumberConverter })
], Slider.prototype, "max", void 0);
__decorate([
    attr({ converter: nullableNumberConverter })
], Slider.prototype, "step", void 0);
__decorate([
    attr
], Slider.prototype, "orientation", void 0);
__decorate([
    attr
], Slider.prototype, "mode", void 0);
